@@ -1,6 +1,6 @@
 /* discover.c -- Hardware-querying utility
  *
- * $Progeny: discover.c 3839 2003-11-17 04:25:01Z dsp $
+ * $Progeny: discover.c 4750 2005-01-27 15:27:56Z licquia $
  *
  * Copyright 2000, 2001, 2002 Progeny Linux Systems, Inc.
  * Copyright 2002 Hewlett-Packard Company
@@ -24,7 +24,7 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <assert.h>
 #include <err.h>
@@ -35,9 +35,11 @@
 #include <ctype.h>
 #include <string.h>
 
-#include <discover.h>
-#include <discover-conf.h>
-#include <discover-xml.h>
+#include <discover/discover.h>
+#include <discover/discover-conf.h>
+#include <discover/discover-xml.h>
+
+#include <discover/device.h>
 
 enum action {
     BUS_SUMMARY,
@@ -248,7 +250,7 @@ print_usage(void)
     puts("\
 usage: discover [--bus-summary] [OPTIONS] [BUS [...]]\n\
        discover --type-summary [OPTIONS] [TYPE [...]]\n\
-       discover --data-path=PATH [--data-version=VERSION] TYPE [...]\n\
+       discover --data-path=PATH [--data-version=VERSION] [TYPE | ID] [...]\n\
        discover --version\n\
        discover --help");
 }
@@ -464,6 +466,7 @@ ll_print(FILE *stream, char *format, linked_list_t *list)
         }
     }
     fputc('\n', stream);
+    free(details);
 }
 
 /******************************************************************************
@@ -523,66 +526,128 @@ type_summary(char *type)
 }
 
 static void
-query_data(char *version, int argc, char **argv)
+process_data_query(discover_device_t *devices, char *version, discover_error_t *status)
 {
-    discover_device_t *devices, *device;
-    discover_error_t *status;
+    discover_device_t *device;
     char *data;
     char *path;
+
+    for (device = devices; device; device = discover_device_get_next(device)) {
+        data_list = ll_new();
+
+        ll_rewind(path_list);
+
+        while((path = ll_next(path_list))) {
+            if (format_string == NULL) {
+                /*
+                 * We'll only test the last path provided on the
+                 * command line if no format string was given.
+                 */
+                path = ll_last(path_list);
+            }
+
+            data = discover_device_get_data(device, path, version, status);
+            check_status(status);
+            if (data) {
+                if (normalize_data) {
+                    data = normalize_text(data);
+                    ll_add(data_list, data);
+                    free(data);
+                } else {
+                    ll_add(data_list, data);
+                }
+            } else {
+                ll_add(data_list, "");
+            }
+
+            if (format_string == NULL) {
+                break;
+            }
+        }
+
+        if (format_string == NULL) {
+            ll_print(stdout, "%s", data_list);
+        } else {
+            ll_print(stdout, format_string, data_list);
+        }
+
+        ll_free(data_list);
+    }
+}
+
+static discover_device_t *
+find_device_by_id(char *id, discover_error_t *status)
+{
+    discover_bus_map_t *busmap;
+    discover_device_t *devices, *device;
+    discover_device_t *result;
     int i;
 
+    /* XXX this is a really inefficient way to do this, but i want to
+       avoid an api change for now */
+    busmap = discover_conf_get_full_bus_map(status);
+    check_status(status);
+    result = NULL;
+    for (i = 0; busmap[i].name; i++) {
+        if (busmap[i].scan_default) {
+            devices = discover_get_devices(i, status);
+            check_status(status);
+            for (device = devices; device; device = discover_device_get_next(device)) {
+                char buf[11];    /* vendorid + modelid */
+                sprintf(buf, "0x%4.4s%4.4s",
+                        discover_device_get_vendor_id(device),
+                        discover_device_get_model_id(device));
+                if (strcmp(buf, id) == 0) {
+                    result = discover_device_new();
+                    discover_device_copy(device, result);
+                    result->next = NULL;
+                    break;
+                }
+            }
+            discover_device_free(devices, 0);
+            memdebug_free(busmap[i]);
+        }
+    }
+
+    return result;
+}
+
+static void
+query_data(char *version, int argc, char **argv)
+{
+    discover_bus_map_t *busmap;
+    discover_device_t *devices, *device;
+    discover_error_t *status;
+    int i, j;
+
     assert(argv != NULL);
 
     status = discover_error_new();
-    for(i = 0; i < argc; i++) {
-        devices = discover_device_find(argv[i], status);
+    if (argc == 0) {
+        /* print information for all busses */
+        busmap = discover_conf_get_full_bus_map(status);
         check_status(status);
-
-        for (device = devices;
-             device;
-             device = discover_device_get_next(device)) {
-
-            data_list = ll_new();
-            ll_rewind(path_list);
-
-            while((path = ll_next(path_list))) {
-                if (format_string == NULL) {
-                    /*
-                     * We'll only test the last path provided on the
-                     * command line if no format string was given.
-                     */
-                    path = ll_last(path_list);
-                }
-
-                data = discover_device_get_data(device, path,
-                                                version, status);
-
+        for (i = 0; busmap[i].name; i++) {
+            if (busmap[i].scan_default) {
+                devices = discover_get_devices(i, status);
                 check_status(status);
-                if (data) {
-                    if (normalize_data) {
-                        data = normalize_text(data);
-                        ll_add(data_list, data);
-                        free(data);
-                    } else {
-                        ll_add(data_list, data);
-                    }
-                } else {
-                    ll_add(data_list, "");
-                }
-
-                if (format_string == NULL) {
-                    break;
-                }
+                process_data_query(devices, version, status);
+                check_status(status);
+                discover_device_free(devices, 0);
+                memdebug_free(busmap[i]);
             }
-
-            if (format_string == NULL) {
-                ll_print(stdout, "%s", data_list);
-            } else {
-                ll_print(stdout, format_string, data_list);
-            }
-
-            ll_free(data_list);
         }
+    }
+    for(i = 0; i < argc; i++) {
+        /* if ARGV[I] starts with "0x", it's a device id */
+        if (strncmp(argv[i], "0x", 2) == 0) {
+            devices = find_device_by_id(argv[i], status);
+        } else {
+            devices = discover_device_find(argv[i], status);
+        }
+        check_status(status);
+        process_data_query(devices, version, status);
+        check_status(status);
         discover_device_free(devices, 0);
     }
     discover_error_free(status);
