@@ -7,7 +7,7 @@
  * storing the XML data during the parsing process.
  */
 
-/* $Progeny: device-xml.c 3839 2003-11-17 04:25:01Z dsp $
+/* $Progeny: device-xml.c 4753 2005-01-29 17:15:42Z licquia $
  *
  * AUTHOR: John R. Daily <jdaily@progeny.com>
  *
@@ -52,7 +52,7 @@
  * pci_devices list in pci.c, and the rest are freed.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 
@@ -65,13 +65,13 @@
 
 #include <expat.h>
 
-#include <discover.h>
-#include <discover-xml.h>
+#include <discover/discover.h>
+#include <discover/discover-xml.h>
 
-#include "load-url.h"
-#include "device.h"
-#include "utils.h"
-#include "stack.h"
+#include <discover/load-url.h>
+#include <discover/device.h>
+#include <discover/utils.h>
+#include <discover/stack.h>
 
 /** Length (including terminating null) of model and vendor ID strings */
 #define IDLEN 5
@@ -172,7 +172,6 @@ create_device(struct context *context, const XML_Char *attrs[])
         }
     }
 
-    assert(busclass != NULL);
     assert(model_id != NULL);
     assert(model_name != NULL);
     assert(vendor_id != NULL);
@@ -185,7 +184,11 @@ create_device(struct context *context, const XML_Char *attrs[])
     new_device = discover_device_new();
     new_device->busclasses = context->busclasses;
     new_device->vendors = context->vendors;
-    new_device->busclass = _discover_xstrdup(busclass);
+    if (busclass) {
+        new_device->busclass = _discover_xstrdup(busclass);
+    } else {
+        new_device->busclass = NULL;
+    }
     new_device->model_id = _discover_xstrdup(model_id);
     new_device->model_name = _discover_xstrdup(model_name);
     new_device->vendor_id = _discover_xstrdup(vendor_id);
@@ -289,7 +292,6 @@ start_element(void *ctx, const XML_Char *name, const XML_Char *attrs[])
         if (strcmp(name, "device") == 0) {
             context->state = DEVICE;
             create_device(context, attrs);
-            context->stack = discover_xml_stack_new();
         }
         break;
 
@@ -343,6 +345,11 @@ end_element(void *ctx, const XML_Char *name)
         stack = context->stack;
         current_data = discover_xml_stack_get(stack);
         device->data = discover_data_get_first(current_data);
+
+        while (discover_xml_stack_pop(&(context->stack))) ;
+        discover_xml_stack_destroy(context->stack);
+        context->stack = NULL;
+
         break;
 
     case DATA:
@@ -443,6 +450,7 @@ discover_xml_merge_device_url(discover_device_t **dlist, char *url,
     context.busclasses = busclasses;
     context.vendors = vendors;
     context.unknown_level = 0;
+    context.stack = NULL;
 
     parser = XML_ParserCreate(NULL);
     XML_SetElementHandler(parser, start_element, end_element);
@@ -618,6 +626,7 @@ discover_xml_get_matching_devices(discover_device_t *xml_devices,
         copy = discover_device_new();
         discover_device_copy(device, copy);
         copy->next = NULL;
+        copy->extra = NULL;
 
         if (last) {
             last->extra = copy;
