@@ -3,7 +3,7 @@
  * @brief Helper utilities
  */
 
-/* $Progeny: utils.c 3839 2003-11-17 04:25:01Z dsp $
+/* $Progeny: utils.c 4368 2004-05-13 19:57:37Z licquia $
  *
  * Copyright 2002 Hewlett-Packard Company
  * Copyright 2002 Progeny Linux Systems, Inc.
@@ -28,7 +28,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 
@@ -39,8 +39,8 @@
 #include <stdlib.h>
 #include <sysexits.h>
 
-#include <discover.h>
-#include "utils.h"
+#include <discover/discover.h>
+#include <discover/utils.h>
 
 void *
 _discover_xmalloc(size_t size)
